/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 1999 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Release: 2.0
 *
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 */

package org.objectweb.jonathan.model;

/**
 * A naming context creates and manages {@link name names}.
 * <p>
 * The role of a naming context is to create and manage names,
 * ensuring that each of the names it manages unambiguously designates a given
 * object interface. 
 * <p>
 * We must first clarify the sense of the terms "object" and "interface" in
 * Jonathan, since they are a bit different
 * from their usual sense in the Java language. The "objects" of
 * interest when developping an application or a library are conceptual entities
 * of the application or library that may be of an arbitrary complexity and
 * granularity. 
 * They can be thought of as components encapsulating a state and information
 * processing. These components, which we call "objects", may be accessed from
 * their environment at specific
 * points that we call <i>interfaces</i>, each of them having its own identity.
 * These objects (components) may be implemented using one or several Java objects.
 * Some of these Java objects, accessible from the component environment, will
 * represent its interfaces. In all the Jonathan documentation, the term "object"
 * designates these components, and the term "interface" or "object interface"
 * designates an identified access point to a component.
 * <p>
 * In Jonathan, naming contexts may take several forms. For instance, 
 * the first example of a naming context is the Java virtual machine itself,
 * that unambiguously designates the objects by their address
 * in memory. The value of an address is not accessible to the Java programmer,
 * but the object identity is manifested to the programmer through an
 * <i>object reference</i>. The problem is that the validity of that reference is
 * limited to the virtual machine, and it cannot be used, e.g., in a persistent or
 * distributed context.
 * <p>
 * Jonathan also defines special types for different kinds of naming contexts:
 * <ul>
 * <li>  {@link org.objectweb.jonathan.apis.binding.NamingContext NamingContext} 
 *       to manage
 *       names of type {@link org.objectweb.jonathan.apis.binding.Identifier 
 *       Identifier}, used
 *       to designate
 *       applicative object interfaces that need to be designated in specific ways
 *       (e.g., distributed or persistent); 
 * <li> {@link org.objectweb.jonathan.apis.protocols.Protocol Protocol} to manage
 *       names of type 
 *       {@link org.objectweb.jonathan.apis.protocols.SessionIdentifier 
 *       SessionIdentifier}, used
 *       to designate interfaces at the protocol level.
 * </ul>
 */
public interface naming_context {

   /**
    * The role of an <tt>export</tt> method is to create a name in a given context.
    * <p>
    * To be usable in a distributed context, an object reference must be
    * <i>exported</i> to another naming context that will create and manage
    * another name for it. If we call the object reference <tt>ref</tt>, we can call
    * the new name <tt>a.ref</tt> to manifest the fact that the new name is built
    * from <tt>ref</tt>. This new name may be re-exported to a third naming context,
    * able to manage names with other properties, and this name may be represented
    * by something like <tt>b.a.ref</tt>. This process may be continued as many
    * times as necessary, thus resulting in reference chains of arbitrary length.
    * <p>
    * No generic <tt>export</tt> operation is defined in Jonathan: such a method
    * may take very different forms, depending on the type of designated object
    * interface, and
    * possible extra parameters. Usually, naming contexts provide one or several
    * <tt>export</tt> methods. However, in some specific cases, like, e.g., if the
    * naming context is a factory that creates objects and directly associates them
    * with an identifier, it is possible that no public <tt>export</tt> method be
    * provided. Export operations may in particular raise 
    * {@link org.objectweb.jonathan.apis.binding.ExportException ExportExceptions} 
    * that indicate that an error occurred in the export process.
    */
   public name export(name name);

   /**
    * <tt>resolve</tt> is the dual operation of
    * {@link #export(name) <tt>export</tt>}: Given a name, say
    * <tt>b.a.ref</tt>, it returns the next name in the referencing chain, in this
    * case, <tt>a.ref</tt>. In the same way, "resolving" <tt>a.ref</tt> results in
    * <tt>ref</tt>.
    * <p>
    * Note that it is not always possible to implement this operation, since the
    * "resolved" name may not have a valid representation in the context where the
    * <tt>resolve</tt> operation is performed. For instance, if <tt>a.ref</tt> is a
    * distributed identifier, and if the <tt>resolve</tt> operation is called in a
    * virtual machine that is not the original virtual machine for <tt>ref</tt>,
    * <tt>ref</tt> cannot be returned. 
    * <p>
    * The <tt>resolve</tt> operation may be borne directly by the name to which it
    * is applied, like e.g., in 
    * {@link org.objectweb.jonathan.apis.binding.Identifier#resolve() this case}.
    */
   public name resolve(name name);
}



